<?php

namespace FiloBlu\ExtMagentoWidget\Model\ResourceModel;

use Magento\Cms\Api\Data\BlockInterface;
use Magento\Framework\DB\Select;
use Magento\Framework\Model\AbstractModel;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\EntityManager\EntityManager;
use Magento\Framework\Model\ResourceModel\Db\Context;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Store\Model\Store;

/**
 * CMS block model
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class Block extends \Magento\Cms\Model\ResourceModel\Block {

    protected $_httpContext;
    protected $_customerSession;

    /**
     * @param Context $context
     * @param StoreManagerInterface $storeManager
     * @param EntityManager $entityManager
     * @param MetadataPool $metadataPool
     * @param string $connectionName
     */
    public function __construct(
    Context $context, StoreManagerInterface $storeManager, EntityManager $entityManager, MetadataPool $metadataPool, \Magento\Framework\App\Http\Context $httpContext, \Magento\Customer\Model\Session $customerSession, $connectionName = null
    ) {

        $this->_httpContext = $httpContext;
        $this->_customerSession = $customerSession;
        parent::__construct($context, $storeManager, $entityManager, $metadataPool, $connectionName);
    }

    /**
     * Retrieve select object for load object data
     *
     * @param string $field
     * @param mixed $value
     * @param \Magento\Cms\Model\Block|AbstractModel $object
     * @return Select
     */
    protected function _getLoadSelect($field, $value, $object) {

        $select = parent::_getLoadSelect($field, $value, $object);

        $_currentCustomerGroupId = "-1";
        if ($this->_httpContext->getValue(\Magento\Customer\Model\Context::CONTEXT_AUTH)) {
            $_currentCustomerGroupId = "-2,";
            $_currentCustomerGroupId = $_currentCustomerGroupId . $this->_customerSession->getCustomer()->getGroupId();
        }

        $select->where('(cms_block.customer_group IS NULL OR cms_block.customer_group IN (' . $_currentCustomerGroupId . '))');

        return $select;
    }
    
    /**
     * Check for unique of identifier of block to selected store(s).
     *
     * @param AbstractModel $object
     * @return bool
     * @SuppressWarnings(PHPMD.BooleanGetMethodName)
     */
    public function getIsUniqueBlockToStores(AbstractModel $object)
    {
        if (!$object->getData('is_active')) {
            return true;
        }
        
        $entityMetadata = $this->metadataPool->getMetadata(BlockInterface::class);
        $linkField = $entityMetadata->getLinkField();

        if ($this->_storeManager->hasSingleStore()) {
            $stores = [Store::DEFAULT_STORE_ID];
        } else {
            $stores = (array)$object->getData('store_id');
        }

        $select = $this->getConnection()->select()
            ->from(['cb' => $this->getMainTable()])
            ->join(
                ['cbs' => $this->getTable('cms_block_store')],
                'cb.' . $linkField . ' = cbs.' . $linkField,
                []
            )
            ->where('cb.identifier = ?', $object->getData('identifier'))
            ->where('cbs.store_id IN (?)', $stores)
            ->where('is_active = ?', 1);
        
        if ($object->getData('customer_group')) {
            $select->where('cb.customer_group IS NULL OR cb.customer_group IN (' . $object->getData('customer_group') . ')');
        }
            
        if ($object->getId()) {
            $select->where('cb.' . $entityMetadata->getIdentifierField() . ' <> ?', $object->getId());
        }

        if ($this->getConnection()->fetchRow($select)) {
            return false;
        }

        return true;
    }
    
}
