<?php

/**
 * Copyright © 2013-2017 Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
/**
 * Widget Instance Resource Model
 *
 * @author      Magento Core Team <core@magentocommerce.com>
 */

namespace FiloBlu\ExtMagentoWidget\Model\ResourceModel\Widget;

use Magento\Framework\Model\AbstractModel;

class Instance extends \Magento\Widget\Model\ResourceModel\Widget\Instance {

    /**
     * Perform actions after object save
     *
     * @param \Magento\Widget\Model\Widget\Instance $object
     * @return $this
     */
    protected function _afterSave(AbstractModel $object) {
        $pageTable = $this->getTable('widget_instance_page');
        $pageLayoutTable = $this->getTable('widget_instance_page_layout');
        $connection = $this->getConnection();

        $select = $connection->select()->from($pageTable, 'page_id')->where('instance_id = ?', (int) $object->getId());
        $pageIds = $connection->fetchCol($select);

        $removePageIds = array_diff($pageIds, $object->getData('page_group_ids'));

        if (is_array($pageIds) && count($pageIds) > 0) {
            $inCond = $connection->prepareSqlCondition('page_id', ['in' => $pageIds]);

            $select = $connection->select()->from($pageLayoutTable, 'layout_update_id')->where($inCond);
            $removeLayoutUpdateIds = $connection->fetchCol($select);

            $connection->delete($pageLayoutTable, $inCond);
            $this->_deleteLayoutUpdates($removeLayoutUpdateIds);
        }

        $this->_deleteWidgetInstancePages($removePageIds);

        if (!$object->getData("disabled")) {
            foreach ($object->getData('page_groups') as $pageGroup) {
                $pageLayoutUpdateIds = $this->_saveLayoutUpdates($object, $pageGroup);
                $data = [
                    'page_group' => $pageGroup['group'],
                    'layout_handle' => $pageGroup['layout_handle'],
                    'block_reference' => $pageGroup['block_reference'],
                    'page_for' => $pageGroup['for'],
                    'entities' => $pageGroup['entities'],
                    'page_template' => $pageGroup['template'],
                ];
                $pageId = $pageGroup['page_id'];
                if (in_array($pageGroup['page_id'], $pageIds)) {
                    $connection->update($pageTable, $data, ['page_id = ?' => (int) $pageId]);
                } else {
                    $connection->insert($pageTable, array_merge(['instance_id' => $object->getId()], $data));
                    $pageId = $connection->lastInsertId($pageTable);
                }
                foreach ($pageLayoutUpdateIds as $layoutUpdateId) {
                    $connection->insert(
                            $pageLayoutTable, ['page_id' => $pageId, 'layout_update_id' => $layoutUpdateId]
                    );
                }
            }
        }

        return \Magento\Framework\Model\ResourceModel\Db\AbstractDb::_afterSave($object);
    }

}
